(function() {
  var extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  if (ELA.Models == null) {
    ELA.Models = {};
  }

  ELA.Models.GraphParams = (function(superClass) {
    extend(GraphParams, superClass);

    GraphParams.prototype.defaults = {
      xScale: 1.1,
      yScale: 1.1,
      xOrigin: 8 + 12 + 8,
      yOrigin: 8 + 12 + 8,
      xOriginAnchor: 'left',
      yOriginAnchor: 'bottom',
      xOriginRatio: null,
      yOriginRatio: null,
      yOffset: 0,
      debug: {},
      xLabelPosition: 'top',
      yLabelPosition: 'right',
      xPanLocked: false,
      yPanLocked: false,
      xPinchLocked: false,
      yPinchLocked: false,
      scaleLink: false,
      xAxisY: 0,
      xAxisScale: {
        type: 'linear'
      },
      yAxisX: 0,
      yAxisScale: {
        type: 'linear'
      }
    };

    function GraphParams() {
      GraphParams.__super__.constructor.apply(this, arguments);
      if ((this.defaults.xOriginRatio != null) && this.defaults.xOriginAnchor === 'right') {
        this.defaults.xOriginRatio = 1 - this.defaults.xOriginRatio;
        this.defaults.xOriginAnchor = 'left';
      }
      if ((this.defaults.yOriginRatio != null) && this.defaults.yOriginAnchor === 'bottom') {
        this.defaults.yOriginRatio = 1 - this.defaults.yOriginRatio;
        this.defaults.yOriginAnchor = 'top';
      }
    }

    GraphParams.prototype.initialize = function() {
      this.once('change:width change:height', this.reset);
      this.on('change:width change:height', this.resize);
      this.on('change:xScale', this.calculateRanges);
      this.on('change:yScale', this.calculateRanges);
      this.calculateRanges();
      this.bindCalculatorEvents();
      this.listenTo(this.app.curves, 'change:selected', this.bindCalculatorEvents);
      return this.listenTo(this.app, 'change:calculators', function() {
        this.calculateRanges();
        return this.bindCalculatorEvents();
      });
    };

    GraphParams.prototype.bindCalculatorEvents = function() {
      var calc, curve, filteredCurves, i, j, len, len1, oldestCurve, ref, results;
      this.stopListening(this.app.previous('calculators'));
      ref = this.app.get('calculators');
      results = [];
      for (i = 0, len = ref.length; i < len; i++) {
        calc = ref[i];
        filteredCurves = this.filteredCurves();
        for (j = 0, len1 = filteredCurves.length; j < len1; j++) {
          curve = filteredCurves[j];
          this.listenTo(calc, "change:" + (curve.get('function')), function() {
            return this.trigger('repaint');
          });
        }
        this.listenTo(calc, 'change:maxX change:xRange', this.calculateRangeX);
        this.listenTo(calc, 'change:maxY change:yRange', this.calculateRangeY);
        oldestCurve = filteredCurves[0];
        if (oldestCurve != null) {
          results.push(this.listenTo(calc, "change:" + (oldestCurve.get('function')), this.calculateRanges));
        } else {
          results.push(void 0);
        }
      }
      return results;
    };

    GraphParams.prototype.reset = function() {
      var xOriginRatio, yOriginRatio;
      if ((xOriginRatio = this.get('xOriginRatio')) != null) {
        this.set({
          xOrigin: this.get('width') * xOriginRatio
        });
      } else if (this.get('xOriginAnchor') === 'left') {
        this.set({
          xOrigin: this.defaults.xOrigin
        });
      } else {
        this.set({
          xOrigin: this.get('width') - this.defaults.xOrigin
        });
      }
      if ((yOriginRatio = this.get('yOriginRatio')) != null) {
        this.set({
          yOrigin: this.get('height') * yOriginRatio
        });
      } else if (this.defaults.yOriginAnchor === 'bottom') {
        this.set({
          yOrigin: this.get('height') - this.defaults.yOrigin
        });
      } else {
        this.set({
          yOrigin: this.defaults.yOrigin
        });
      }
      if (this.get('scaleLink')) {
        if (this.get('width') / this.maxRangeX(null, 1) < this.params.get('height') / this.maxRangeY(null, 1)) {
          return this.set({
            yScale: this.linkedYScale(xScale)
          });
        } else {
          return this.set({
            xScale: this.linkedXScale(yScale)
          });
        }
      } else {
        return this.set({
          xScale: this.defaults.xScale,
          yScale: this.defaults.yScale
        });
      }
    };

    GraphParams.prototype.resize = function() {
      var previous, previousXOrigin, previousYOrigin;
      previous = this.previousAttributes();
      if ((previous.width != null) && (previous.height != null)) {
        previousXOrigin = previous.xOrigin != null ? previous.xOrigin : this.get('xOrigin');
        previousYOrigin = previous.yOrigin != null ? previous.yOrigin : this.get('yOrigin');
        this.set({
          xOrigin: previousXOrigin * this.get('width') / previous.width,
          yOrigin: previousYOrigin * this.get('height') / previous.height
        });
        switch (this.get('scaleLink')) {
          case 'x':
            return this.set({
              xScale: this.linkedXScale()
            });
          case 'y':
            return this.set({
              yScale: this.linkedYScale()
            });
        }
      } else {
        return this.reset();
      }
    };

    GraphParams.prototype.calculateRangeX = function() {
      return this.set({
        xRange: this.maxRangeX()
      });
    };

    GraphParams.prototype.calculateRangeY = function() {
      return this.set({
        yRange: this.maxRangeY()
      });
    };

    GraphParams.prototype.calculateRanges = function() {
      this.calculateRangeX();
      return this.calculateRangeY();
    };

    GraphParams.prototype.maxRangeX = function(func, xScale) {
      var calc, ranges, ref;
      if (xScale == null) {
        xScale = this.get('xScale');
      }
      if (func == null) {
        func = (ref = this.get('axisLabelingForCurve')) != null ? ref.get('function') : void 0;
      }
      ranges = (function() {
        var i, len, ref1, results;
        ref1 = this.app.get('calculators');
        results = [];
        for (i = 0, len = ref1.length; i < len; i++) {
          calc = ref1[i];
          results.push(calc.xRange(func));
        }
        return results;
      }).call(this);
      return Math.max.apply(Math, _.compact(ranges)) * xScale;
    };

    GraphParams.prototype.maxRangeY = function(func, yScale) {
      var calc, ranges, ref;
      if (yScale == null) {
        yScale = this.get('yScale');
      }
      if (func == null) {
        func = (ref = this.get('axisLabelingForCurve')) != null ? ref.get('function') : void 0;
      }
      ranges = (function() {
        var i, len, ref1, results;
        ref1 = this.app.get('calculators');
        results = [];
        for (i = 0, len = ref1.length; i < len; i++) {
          calc = ref1[i];
          results.push(calc.yRange(func));
        }
        return results;
      }).call(this);
      return Math.max.apply(Math, _.compact(ranges)) * yScale;
    };

    GraphParams.prototype.linkedXScale = function(yScale) {
      return (this.get('width') / this.maxRangeX(null, 1)) / (this.get('height') / this.maxRangeY(null, yScale));
    };

    GraphParams.prototype.linkedYScale = function(xScale) {
      return (this.get('height') / this.maxRangeY(null, 1)) / (this.get('width') / this.maxRangeX(null, xScale));
    };

    GraphParams.prototype.filteredCurves = function() {
      var curves, graphCurves;
      curves = this.app.curves.history;
      if (graphCurves = this.get('curves')) {
        return _.filter(curves, function(curve) {
          return graphCurves.indexOf(curve.get('function')) >= 0;
        });
      } else {
        return curves;
      }
    };

    GraphParams.prototype.sortedCurves = function() {
      return _.sortBy(this.filteredCurves(), function(curve) {
        return curve.get('zIndex');
      });
    };

    GraphParams.prototype.serialize = function() {
      return {
        xScale: this.get('xScale'),
        yScale: this.get('yScale'),
        xOriginRatio: this.get('xOrigin') / this.get('width'),
        yOriginRatio: this.get('yOrigin') / this.get('height')
      };
    };

    GraphParams.prototype.deserialize = function(attributes) {
      return this.set({
        xScale: attributes.xScale,
        yScale: attributes.yScale,
        xOrigin: this.get('width') * attributes.xOriginRatio,
        yOrigin: this.get('height') * attributes.yOriginRatio
      });
    };

    return GraphParams;

  })(ELA.Models.CanvasParams);

}).call(this);
