(function() {
  var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  if (ELA.Views == null) {
    ELA.Views = {};
  }

  ELA.Views.Canvas = (function(superClass) {
    extend(Canvas, superClass);

    Canvas.Params = ELA.Models.CanvasParams;

    Canvas.prototype.tagName = 'canvas';

    Canvas.prototype.animationFrameRequested = false;

    function Canvas() {
      this.requestRepaint = bind(this.requestRepaint, this);
      this.readCanvasResolution = bind(this.readCanvasResolution, this);
      var fontWeight;
      Canvas.__super__.constructor.apply(this, arguments);
      fontWeight = window.devicePixelRatio > 1 ? 300 : 400;
      this.defaultFont = fontWeight + " 12px Roboto";
      this.largeBoldFont = (fontWeight * 2) + " 20px Roboto";
    }

    Canvas.prototype.initialize = function(options) {
      if (options == null) {
        options = {};
      }
      this.params = options.params;
      this.setCanvasResolution();
      $(window).on('resize', this.readCanvasResolution);
      this.listenTo(this.model, 'change:calculators', this.readCanvasResolution);
      this.listenTo(this.params, 'change:width change:height', this.setCanvasResolution);
      return this.listenTo(this.params, 'repaint', this.requestRepaint);
    };

    Canvas.prototype.remove = function() {
      Canvas.__super__.remove.apply(this, arguments);
      return $(window).off('resize', this.readCanvasResolution);
    };

    Canvas.prototype.readCanvasResolution = function() {
      var $parent;
      $parent = this.$el.parent();
      if ($parent.length > 0) {
        return this.params.set({
          width: $parent[0].clientWidth,
          height: $parent[0].clientHeight
        });
      }
    };

    Canvas.prototype.setCanvasResolution = function() {
      var backingStoreRatio, context, devicePixelRatio, height, width;
      width = this.params.get('width');
      height = this.params.get('height');
      context = this.el.getContext('2d');
      devicePixelRatio = window.devicePixelRatio || 1;
      backingStoreRatio = context.webkitBackingStorePixelRatio || context.mozBackingStorePixelRatio || context.msBackingStorePixelRatio || context.oBackingStorePixelRatio || context.backingStorePixelRatio || 1;
      context.pixelRatio = this.pixelRatio = devicePixelRatio / backingStoreRatio;
      this.$el.css({
        width: width,
        height: height
      });
      this.$el.attr({
        width: width * this.pixelRatio,
        height: height * this.pixelRatio
      });
      return context.scale(this.pixelRatio, this.pixelRatio);
    };

    Canvas.prototype.requestRepaint = function() {
      if (!this.animationFrameRequested) {
        this.animationFrameRequested = true;
        return requestAnimationFrame((function(_this) {
          return function() {
            _this.animationFrameRequested = false;
            return _this.render();
          };
        })(this));
      }
    };

    return Canvas;

  })(Backbone.Poised.View);

}).call(this);
