(function() {
  var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  if (ELA.Views == null) {
    ELA.Views = {};
  }

  ELA.Views.Legend = (function(superClass) {
    extend(Legend, superClass);

    function Legend() {
      this.render = bind(this.render, this);
      this.curves = bind(this.curves, this);
      this.renderSimpleCurveColumn = bind(this.renderSimpleCurveColumn, this);
      this.renderValueCurveColumn = bind(this.renderValueCurveColumn, this);
      this.renderValueHeaderColumn = bind(this.renderValueHeaderColumn, this);
      return Legend.__super__.constructor.apply(this, arguments);
    }

    Legend.prototype.className = 'legend';

    Legend.prototype.valueCurveColumnTemplate = _.template('<div class="values col<%= activeClass %>" data-index="<%= curveIndex %>" style="color: <%= strokeColor %>; <%= borderStyle %>"> <div class="label"><%= label %></div> </div>');

    Legend.prototype.simpleCurveColumnTemplate = _.template('<div class="curve"> <div class="line" style="border-color: <%= strokeColor %>"></div> <%= label %> </div>');

    Legend.prototype.useValueAtRange = false;

    Legend.prototype.initialize = function(options) {
      this.displayParams = options.displayParams;
      this.listenTo(this.displayParams, 'change:axisLabelingForCurve', this.render);
      this.listenTo(this.model, 'change:calculators', (function(_this) {
        return function() {
          _this.bindCalculatorEvents();
          return _this.render();
        };
      })(this));
      this.listenTo(this.model.curves, 'change:selected', this.render);
      this.bindCalculatorEvents();
      if (options.valueAttribute) {
        this.valueAttribute = options.valueAttribute;
      }
      this._curves = options.curves;
      if (this.valueAttribute) {
        return this.listenTo(this.model, "change:" + this.valueAttribute, this.render);
      }
    };

    Legend.prototype.events = {
      'click .values.col': 'selectCurveForAxisLabeling'
    };

    Legend.prototype.selectCurveForAxisLabeling = function(e) {
      var index;
      index = parseInt($(e.currentTarget).data('index'));
      return this.displayParams.set({
        axisLabelingForCurve: this.model.curves.at(index)
      });
    };

    Legend.prototype.bindCalculatorEvents = function() {
      var calc, curve, j, len, ref1, results;
      this.stopListening(this.model.previous('calculators'));
      ref1 = this.model.get('calculators');
      results = [];
      for (j = 0, len = ref1.length; j < len; j++) {
        calc = ref1[j];
        results.push((function() {
          var k, len1, ref2, results1;
          ref2 = this.curves();
          results1 = [];
          for (k = 0, len1 = ref2.length; k < len1; k++) {
            curve = ref2[k];
            results1.push(this.listenTo(calc, "change:" + (curve.get('function')), this.render));
          }
          return results1;
        }).call(this));
      }
      return results;
    };

    Legend.prototype.renderValueHeaderColumn = function() {};

    Legend.prototype.renderValueCurveColumn = function(curve) {
      var $curve, calc, diff, func, i, isActive, j, label, len, ref, ref1, ref2, unitValue, val;
      if (!curve.showInLegend()) {
        return;
      }
      func = this.calculatorFunction(curve);
      ref = (ref1 = this.calcs[0]) != null ? typeof ref1[func] === "function" ? ref1[func](this.range) : void 0 : void 0;
      isActive = curve === this.labelingCurve;
      $curve = $(this.valueCurveColumnTemplate({
        activeClass: isActive ? ' active' : '',
        curveIndex: this.model.curves.indexOf(curve),
        strokeColor: curve.strokeStyle(),
        borderStyle: isActive ? "border-color: " + (curve.strokeStyle()) : void 0,
        label: this.Present(curve).fullLabel()
      }));
      ref2 = this.calcs;
      for (i = j = 0, len = ref2.length; j < len; i = ++j) {
        calc = ref2[i];
        val = calc[func](this.range);
        if (val != null) {
          unitValue = this.Present(curve).unitValue(val);
          if (_.isArray(unitValue)) {
            unitValue = _.compact(unitValue);
            unitValue = _.map(unitValue, function(v) {
              return v.toFixed(2);
            });
            label = "{" + (unitValue.join(', ')) + "}";
          } else {
            label = unitValue.toFixed(2);
          }
          if (ref && i > 0) {
            diff = (val / ref * 100) - 100;
            label += " (" + (diff.toFixed(2)) + "%)";
          }
          $curve.append("<div>" + label + "</div>");
        } else {
          $curve.append("<div>" + (t('legend.notAvailable')) + "</div>");
        }
      }
      return this.$wrapper.append($curve);
    };

    Legend.prototype.renderSimpleCurveColumn = function(curve, curveIndex) {
      if (!curve.showInLegend()) {
        return;
      }
      return this.$el.append($(this.simpleCurveColumnTemplate({
        strokeColor: curve.strokeStyle(),
        label: this.Present(curve).label()
      })));
    };

    Legend.prototype.calculatorFunction = function(curve) {
      return (curve.get('function')) + "_value";
    };

    Legend.prototype.curves = function() {
      var curves;
      curves = this.model.curves.whereInHistory();
      if (this._curves) {
        return _.filter(curves, (function(_this) {
          return function(curve) {
            return _this._curves.indexOf(curve.get('function')) >= 0;
          };
        })(this));
      } else {
        return curves;
      }
    };

    Legend.prototype.render = function() {
      var scrollLeft;
      if (this.valueAttribute) {
        this.range = this.model.get(this.valueAttribute);
        if (this.range != null) {
          this.labelingCurve = this.displayParams.get('axisLabelingForCurve');
          this.calcs = this.model.get('calculators');
          this.$wrapper = $('<div class="values-at-range scroll-x">');
          this.renderValueHeaderColumn();
          _.each(this.curves(), this.renderValueCurveColumn);
          scrollLeft = this.$el.find('.scroll-x').scrollLeft();
          this.$el.html(this.$wrapper);
          this.$el.find('.scroll-x').scrollLeft(scrollLeft);
        }
      } else {
        this.$el.empty().addClass('legend-simple');
        _.each(this.curves(), this.renderSimpleCurveColumn);
      }
      return this;
    };

    return Legend;

  })(Backbone.Poised.View);

}).call(this);
