(function() {
  var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  Backbone.Poised.BaseControl = (function(superClass) {
    extend(BaseControl, superClass);

    function BaseControl() {
      this.renderVisibility = bind(this.renderVisibility, this);
      this.clone = bind(this.clone, this);
      this.render = bind(this.render, this);
      this.hintValidation = bind(this.hintValidation, this);
      this.initialize = bind(this.initialize, this);
      return BaseControl.__super__.constructor.apply(this, arguments);
    }

    BaseControl.prototype.template = _.template('<div class="info"><label><%= label %></label><div class="hint"></div><div class="input"></div></div>');

    BaseControl.prototype.initialize = function(options) {
      var i, len, p, ref;
      if (options == null) {
        options = {};
      }
      this.initOptions = options;
      if (options.model == null) {
        throw new Error('Missing `model` option');
      }
      if (options.attribute == null) {
        throw new Error('Missing `attribute` option');
      }
      this.attribute = options.attribute;
      this.label = options.label;
      this.options = _.omit(options, 'el');
      this.visibilityParameters = this.options.visibility ? this.options.visibility.slice(0) : [];
      this.visibilityCallback = this.visibilityParameters.pop();
      ref = this.visibilityParameters;
      for (i = 0, len = ref.length; i < len; i++) {
        p = ref[i];
        this.model.on("change:" + p, this.renderVisibility);
      }
      if (this.options.validate) {
        this.model.on('validated', this.hintValidation);
      }
      this.on('liveChangeStart', (function(_this) {
        return function() {
          if (_this.parentView != null) {
            return _this.parentView.trigger('controlLiveChangeStart', _this);
          }
        };
      })(this));
      this.on('liveChange', (function(_this) {
        return function() {
          if (_this.parentView != null) {
            return _this.parentView.trigger('controlLiveChange', _this);
          }
        };
      })(this));
      return this.on('liveChangeEnd', (function(_this) {
        return function() {
          if (_this.parentView != null) {
            return _this.parentView.trigger('controlLiveChangeEnd', _this);
          }
        };
      })(this));
    };

    BaseControl.prototype.hintValidation = function(isValid, model, errors) {
      var attributeValid;
      attributeValid = this.model.isValid(this.attribute);
      if (errors !== this.lastErrors) {
        this.$el.toggleClass('invalid', !attributeValid);
        this.$hint = this.$('.hint');
        this.$hint.attr({
          'data-error': errors[this.attribute]
        });
      }
      return this.lastErrors = errors;
    };

    BaseControl.prototype.render = function() {
      this.$el.attr('class', "poised control " + this.attribute);
      this.renderVisibility();
      this.$el.html(this.template({
        label: this.label || this.loadLocale("formFields." + this.attribute + ".label", {
          defaultValue: this.attribute.toLabel()
        })
      }));
      this.$input = this.$el.find('.input');
      return this;
    };

    BaseControl.prototype.clone = function(options) {
      if (options == null) {
        options = {};
      }
      return new this.__proto__.constructor(_.defaults(options, this.initOptions));
    };

    BaseControl.prototype.renderVisibility = function() {
      var isVisible, p, parameters;
      if (!this.visibilityCallback) {
        return;
      }
      parameters = (function() {
        var i, len, ref, results;
        ref = this.visibilityParameters;
        results = [];
        for (i = 0, len = ref.length; i < len; i++) {
          p = ref[i];
          results.push(this.model.get(p));
        }
        return results;
      }).call(this);
      isVisible = this.visibilityCallback.call(parameters);
      return this.$el.toggleClass('hidden', !isVisible);
    };

    return BaseControl;

  })(Backbone.Poised.View);

}).call(this);
