(function() {
  var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  Backbone.Poised.Textfield = (function(superClass) {
    extend(Textfield, superClass);

    function Textfield() {
      this.render = bind(this.render, this);
      this.clearInputValue = bind(this.clearInputValue, this);
      this.readInput = bind(this.readInput, this);
      this.handleKeyUp = bind(this.handleKeyUp, this);
      this._updateValue = bind(this._updateValue, this);
      this.initialize = bind(this.initialize, this);
      return Textfield.__super__.constructor.apply(this, arguments);
    }

    Textfield.prototype.className = 'poised textfield';

    Textfield.prototype.events = {
      'focusin input': 'clearInputValue',
      'focusout input': 'readInput',
      'keyup input': 'handleKeyUp'
    };

    Textfield.prototype.initialize = function(options) {
      var ref;
      if (options == null) {
        options = {};
      }
      if (options.model == null) {
        throw new Error('Missing `model` option');
      }
      if (options.attribute == null) {
        throw new Error('Missing `attribute` option');
      }
      this.attribute = options.attribute;
      this.options = _.chain(options).pick('type', 'placeholder', 'autofocus', 'clearOnFocus', 'stepSize', 'precision', 'minValue', 'maxValue', 'unit', 'validate').defaults({
        type: 'text',
        autofocus: false,
        clearOnFocus: false,
        minValue: null,
        maxValue: null,
        validate: true
      }).value();
      if ((this.options.stepSize != null) && (this.options.precision == null)) {
        this.options.precision = _.find([0, 1, 2, 3], (function(_this) {
          return function(i) {
            return _this.options.stepSize * Math.pow(10, i) >= 1;
          };
        })(this));
      }
      if (_.isArray(options.range)) {
        ref = options.range, this.options.minValue = ref[0], this.options.maxValue = ref[1];
      }
      return this.listenTo(this.model, "change:" + this.attribute, this.setValue);
    };

    Textfield.prototype.limit = function(val) {
      if (this.options.minValue != null) {
        val = Math.max(val, this.options.minValue);
      }
      if (this.options.maxValue != null) {
        val = Math.min(val, this.options.maxValue);
      }
      return val;
    };

    Textfield.prototype._updateValue = function(value, options) {
      var valueString;
      if (value == null) {
        value = this.model.get(this.attribute);
      }
      if (options == null) {
        options = {};
      }
      if (this.options.type === 'number') {
        if (typeof value === 'string') {
          if (value.match(/^[+\-]?\d+(\.\d+)?$/i) != null) {
            value = this.limit(parseFloat(value));
          } else {
            value = this.model.get(this.attribute);
          }
        }
        if (this.options.precision != null) {
          valueString = value.toFixed(this.options.precision);
        }
      }
      this.$input.val(valueString || value);
      if (options.updateModel !== false) {
        this.model.set(this.attribute, value);
        if ((this.model.validate != null) && this.options.validate) {
          return this.model.validate(this.attribute);
        }
      }
    };

    Textfield.prototype.setValue = function() {
      if (this.$input.is(':focus')) {
        return;
      }
      return this._updateValue(void 0, {
        updateModel: false
      });
    };

    Textfield.prototype.handleKeyUp = function(e) {
      if (e.keyCode === 13) {
        return this.readInput();
      }
    };

    Textfield.prototype.readInput = function() {
      this._updateValue(this.$input.val());
      return this.trigger('changeValue');
    };

    Textfield.prototype.clearInputValue = function() {
      if (this.options.clearOnFocus) {
        return this.$input.val('');
      }
    };

    Textfield.prototype.render = function() {
      var inputAttributes;
      this.$el.empty();
      inputAttributes = {
        "class": 'poised',
        name: this.attribute,
        type: this.options.type,
        placeholder: this.options.placeholder,
        autofocus: this.options.autofocus
      };
      if (this.options.type === 'number') {
        inputAttributes.min = this.options.minValue;
        inputAttributes.max = this.options.maxValue;
        inputAttributes.step = this.options.stepSize || 'any';
      }
      this.$input = $('<input />', inputAttributes).appendTo(this.$el);
      if (this.options.unit) {
        this.$el.append($('<abbr/>', {
          text: this.options.unit
        }));
      }
      this._updateValue();
      return this;
    };

    return Textfield;

  })(Backbone.View);

}).call(this);
