(function() {
  var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  Backbone.Poised.List.SelectableItem = (function(superClass) {
    extend(SelectableItem, superClass);

    function SelectableItem() {
      this.render = bind(this.render, this);
      this.updateCheckboxState = bind(this.updateCheckboxState, this);
      this.toggleSingleSelection = bind(this.toggleSingleSelection, this);
      this.updateModelSelection = bind(this.updateModelSelection, this);
      this.updateCheckbox = bind(this.updateCheckbox, this);
      return SelectableItem.__super__.constructor.apply(this, arguments);
    }

    SelectableItem.prototype.events = _.defaults({
      'change [type=checkbox]': 'updateModelSelection',
      'tap': 'updateModelSelection',
      'press': 'toggleSingleSelection'
    }, Backbone.Poised.List.Item.prototype.events);

    SelectableItem.prototype.selectedAttribute = 'selected';

    SelectableItem.prototype.hammerjs = {
      recognizers: [
        [
          Hammer.Rotate, {
            enable: false
          }
        ], [
          Hammer.Pinch, {
            enable: false
          }, ['rotate']
        ], [
          Hammer.Swipe, {
            enable: false
          }
        ], [
          Hammer.Pan, {
            enable: false
          }, ['swipe']
        ], [
          Hammer.Tap, {
            threshold: 5
          }
        ], [Hammer.Press]
      ]
    };

    SelectableItem.prototype.initialize = function(options) {
      SelectableItem.__super__.initialize.apply(this, arguments);
      this.options.singleSelect = options.singleSelect === true;
      return this.model.on("change:" + this.selectedAttribute, this.updateCheckboxState);
    };

    SelectableItem.prototype.updateCheckbox = function() {
      return this.$checkbox.attr({
        checked: this.model.get(this.selectedAttribute)
      });
    };

    SelectableItem.prototype.updateModelSelection = function(e) {
      this.model.set(this.selectedAttribute, !this.model.get(this.selectedAttribute));
      this.model.collection.singleSelection = void 0;
      return this.model.collection.selectedBeforeSingleSelection = void 0;
    };

    SelectableItem.prototype.toggleSingleSelection = function() {
      var collection, filter;
      if (this.options.singleSelect) {
        collection = this.model.collection;
        if (collection.singleSelection === this.model) {
          collection.each((function(_this) {
            return function(model) {
              var previousSelection, previouslySelected;
              previousSelection = collection.selectedBeforeSingleSelection;
              previouslySelected = previousSelection.indexOf(model) > -1;
              return model.set(_this.selectedAttribute, previouslySelected);
            };
          })(this));
          return collection.singleSelection = void 0;
        } else {
          filter = {};
          filter[this.selectedAttribute] = true;
          if (collection.selectedBeforeSingleSelection == null) {
            collection.selectedBeforeSingleSelection = collection.where(filter);
          }
          collection.singleSelection = this.model;
          return collection.each((function(_this) {
            return function(model) {
              return model.set(_this.selectedAttribute, model === _this.model);
            };
          })(this));
        }
      }
    };

    SelectableItem.prototype.updateCheckboxState = function() {
      return this.$checkbox.prop('checked', this.model.get(this.selectedAttribute));
    };

    SelectableItem.prototype.render = function() {
      SelectableItem.__super__.render.apply(this, arguments);
      this.$checkbox = $('<input>', {
        type: 'checkbox',
        name: "model_" + this.model.cid
      });
      this.updateCheckboxState();
      this.$buttons.append(this.$checkbox);
      this.$buttons.append($('<label>', {
        "for": "model_" + this.model.cid,
        "class": 'checkbox'
      }));
      return this;
    };

    return SelectableItem;

  })(Backbone.Poised.List.Item);

}).call(this);
